#include <avr/io.h>
#include "i2cmaster.h"
#include "ssd1306.h"

// initializing oled
// 
void oled_init(void)
{
    unsigned char ret;
    
    i2c_init();             // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {            // I2C problem 
        i2c_stop();
        PORTB=0x00;         // LED zapneme = error
    } else {                // I2c komunikuje tak zacneme init LCD
	i2c_write(0x00);    // command to end i2c session

        i2c_write(0xa8);    // Set Multiplex Ratio 3f = 64MUX
        i2c_write(0x3f);

        i2c_write(0xd3);    // Set Display Offset 00
        i2c_write(0x00);

        i2c_write(0x40);    // Set Display Start Line 

        i2c_write(0xa1);    // Set Segment Re-map

        i2c_write(0xc8);    // Set COM Output Scan Direction

        i2c_write(0xda);    // Set COM Pins Hardware Configuration
        i2c_write(0x12);

        i2c_write(0x81);    // Set Contrast Control 7f default
        i2c_write(0x7f);

        i2c_write(0xa4);    // Entre Display ON

        i2c_write(0xa6);    // Set Normal/Inverse Display

        i2c_write(0xd5);    // Set Display Clock Divide Ratio/Oscillator Frequency
        i2c_write(0x80);

        i2c_stop();                            
    }
}

/* enable oled
 * enable charge pump and display
 */
void oled_enable(void)
{
    unsigned char ret;

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak zapneme zobrazovanie 
	i2c_write(0x00);                // command to end i2c session

        i2c_write(0x8d);                // Enable charge pump
        i2c_write(0x14);

        i2c_write(0xaf);                // Set Display ON
        i2c_stop();
    }
}


/* disable oled (stand by)
 * disable display and charge pump
 */
void oled_disable(void)
{
    unsigned char ret;

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak vypneme zobrazovanie
        i2c_write(0x00);                // command to end i2c session
        i2c_write(0xae);                // Set Display OFF

        i2c_write(0x8d);                // Disable charge pump
        i2c_write(0x10);
        i2c_stop();
    }
}


// enable/disable inversion
void oled_inverse(unsigned char inverse)
{
    unsigned char ret;

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak posleme priaz
        i2c_write(0x80);                // single command
        if ( inverse )
                i2c_write(0xa6);          // Set Normal display
        else
                i2c_write(0xa7);          // Set Inverse display
        i2c_stop();
    }       
}

/* enable/disable displaing GDRAM contens
 * Enable  - displaing actal GDRAM contens (displaing changes)
 * Disable - d'not display GDRAM changes.
 */
void oled_display(unsigned char display)
{
    unsigned char ret;

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak posleme prikaz
        i2c_write(0x80);                // single command
        if ( display )
                i2c_write(0xa5);          // ON
        else
                i2c_write(0xa4);          // GDRAM
        i2c_stop();
    }
}


/* define area in GDRAM where you can write
 * pbegin - start page (row) 0-7
 * pend   - end page (row) 0-7
 *          pend > pbegin else exit false
 * cbegin - start column 0-127
 * cend   - top column 0-127
 *          cend > cbegin else exit false
 * vertical - vertical filling GDRAM
 */
unsigned char oled_area(unsigned char pbegin, unsigned char pend, unsigned char cbegin, unsigned char cend, unsigned char vertical)
{
    unsigned char ret;

    if ((cbegin > cend) || (pbegin > pend))
	return 1;
    else
        {
        i2c_init();                         // init I2C interface
        ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
        if ( ret ) {                        // I2C problem
            i2c_stop();
            PORTB=0x00;                     // LED zapneme = error
        } else {                            // I2c komunikuje tak zacneme init LCD
            i2c_write(0x00);                // command to end i2c session

            i2c_write(0x20);                // set addresing mode 
            if (vertical)	
	            i2c_write(0x01);        // vertical addressing
            else
                    i2c_write(0x00);        // horizotal addresing

            i2c_write(0x21);                // set columns range
            i2c_write(cbegin&0x7f);         // start colmns
            i2c_write(cend&0x7f);           // end columns

            i2c_write(0x22);                // set pafe range
            i2c_write(pbegin&0x07);         // start page
            i2c_write(pend&0x07);           // end page
        }
       return 0;
    }
}

/* send data to display
 * *address - pointer to array
 * size - size of array
 */
void oled_data( unsigned char *address, unsigned int size)
{
    unsigned char ret;
    unsigned int i;

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak vypneme zobrazovanie
        i2c_write(0x40);               // data to end i2c session
	for(i=0;i<size;i++)
            i2c_write(*(address+i));
        i2c_stop();
    }
}

/* clear screen
 * WARNING!! comand redefine oled area
 */
void oled_clr(void)
{
    unsigned char ret;
    unsigned int i;

    oled_area(0,7,0,127,0);

    i2c_init();                         // init I2C interface
    ret = i2c_start(DevLCD+I2C_WRITE);  // set device address and write mode
    if ( ret ) {                        // I2C problem
        i2c_stop();
        PORTB=0x00;                     // LED zapneme = error
    } else {                            // I2c komunikuje tak vypneme zobrazovanie
        i2c_write(0x40);               // data to end i2c session
        for(i=0;i<1024;i++)
            i2c_write(0x00);
        i2c_stop();
    }
}    
